@@ -1,4 +1,4 @@
-# $Id: __init__.py 4564 2006-05-21 20:44:42Z wiemann $
+# $Id: __init__.py 5618 2008-07-28 08:37:32Z strank $
 # Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
@@ -14,7 +14,7 @@ __docformat__ = 'reStructuredText'
 _languages = {}
 
 def get_language(language_code):
-    if _languages.has_key(language_code):
+    if language_code in _languages:
         return _languages[language_code]
     module = __import__(language_code, globals(), locals())
     _languages[language_code] = module
