@@ -1,7 +1,5 @@
-# Author: David Goodger
-# Contact: goodger@users.sourceforge.net
-# Revision: $Revision: 1.16 $
-# Date: $Date: 2004/06/17 21:46:50 $
+# $Id: statemachine.py 6314 2010-04-26 10:04:17Z milde $
+# Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
 """
@@ -110,7 +108,8 @@ __docformat__ = 'restructuredtext'
 
 import sys
 import re
-from types import SliceType as _SliceType
+import types
+import unicodedata
 
 
 class StateMachine:
@@ -177,7 +176,7 @@ class StateMachine:
         self.states = None
 
     def run(self, input_lines, input_offset=0, context=None,
-            input_source=None):
+            input_source=None, initial_state=None):
         """
         Run the state machine on `input_lines`. Return results (a list).
 
@@ -197,6 +196,7 @@ class StateMachine:
           of the file.
         - `context`: application-specific storage.
         - `input_source`: name or path of source of `input_lines`.
+        - `initial_state`: name of initial state.
         """
         self.runtime_init()
         if isinstance(input_lines, StringList):
@@ -205,7 +205,7 @@ class StateMachine:
             self.input_lines = StringList(input_lines, source=input_source)
         self.input_offset = input_offset
         self.line_offset = -1
-        self.current_state = self.initial_state
+        self.current_state = initial_state or self.initial_state
         if self.debug:
             print >>sys.stderr, (
                 '\nStateMachine.run: input_lines (line_offset=%s):\n| %s'
@@ -356,11 +356,44 @@ class StateMachine:
         """Return line number of current line (counting from 1)."""
         return self.line_offset + self.input_offset + 1
 
+    def get_source_and_line(self, lineno=None):
+        """Return (source, line) tuple for current or given line number.
+
+        Looks up the source and line number in the `self.input_lines`
+        StringList instance to count for included source files.
+
+        If the optional argument `lineno` is given, convert it from an
+        absolute line number to the corresponding (source, line) pair.
+        """
+        if lineno is None:
+            offset = self.line_offset
+        else:
+            offset = lineno - self.input_offset - 1
+        try:
+            src, srcoffset = self.input_lines.info(offset)
+            srcline = srcoffset + 1
+        except (TypeError):
+            # line is None if index is "Just past the end"
+            src, line = self.get_source_and_line(offset + self.input_offset)
+            return src, line + 1
+        except (IndexError): # `offset` is off the list
+            src, srcline = None, None
+            # raise AssertionError('cannot find line %d in %s lines' %
+            #                      (offset, len(self.input_lines)))
+            #                      # list(self.input_lines.lines())))
+        # assert offset == srcoffset, str(self.input_lines)
+        # print "get_source_and_line(%s):" % lineno,
+        # print offset + 1, '->', src, srcline
+        # print self.input_lines
+        return (src, srcline)
+
     def insert_input(self, input_lines, source):
         self.input_lines.insert(self.line_offset + 1, '',
-                                source='internal padding')
+                                source='internal padding after '+source,
+                                offset=len(input_lines))
         self.input_lines.insert(self.line_offset + 1, '',
-                                source='internal padding')
+                                source='internal padding before '+source,
+                                offset=-1)
         self.input_lines.insert(self.line_offset + 2,
                                 StringList(input_lines, source))
 
@@ -378,7 +411,7 @@ class StateMachine:
             self.next_line(len(block) - 1)
             return block
         except UnexpectedIndentationError, error:
-            block, source, lineno = error
+            block, source, lineno = error.args
             self.next_line(len(block) - 1) # advance to last line of block
             raise
 
@@ -411,7 +444,7 @@ class StateMachine:
                   % (state.__class__.__name__, transitions))
         for name in transitions:
             pattern, method, next_state = state.transitions[name]
-            match = self.match(pattern)
+            match = pattern.match(self.line)
             if match:
                 if self.debug:
                     print >>sys.stderr, (
@@ -426,14 +459,6 @@ class StateMachine:
                       % state.__class__.__name__)
             return state.no_match(context, transitions)
 
-    def match(self, pattern):
-        """
-        Return the result of a regular expression match.
-
-        Parameter `pattern`: an `re` compiled regular expression.
-        """
-        return pattern.match(self.line)
-
     def add_state(self, state_class):
         """
         Initialize & add a `state_class` (`State` subclass) object.
@@ -442,7 +467,7 @@ class StateMachine:
         added.
         """
         statename = state_class.__name__
-        if self.states.has_key(statename):
+        if statename in self.states:
             raise DuplicateStateError(statename)
         self.states[statename] = state_class(self, self.debug)
 
@@ -630,9 +655,9 @@ class State:
         Exceptions: `DuplicateTransitionError`, `UnknownTransitionError`.
         """
         for name in names:
-            if self.transitions.has_key(name):
+            if name in self.transitions:
                 raise DuplicateTransitionError(name)
-            if not transitions.has_key(name):
+            if name not in transitions:
                 raise UnknownTransitionError(name)
         self.transition_order[:0] = names
         self.transitions.update(transitions)
@@ -645,7 +670,7 @@ class State:
 
         Exception: `DuplicateTransitionError`.
         """
-        if self.transitions.has_key(name):
+        if name in self.transitions:
             raise DuplicateTransitionError(name)
         self.transition_order[:0] = [name]
         self.transitions[name] = transition
@@ -765,7 +790,7 @@ class StateMachineWS(StateMachine):
     `StateMachine` subclass specialized for whitespace recognition.
 
     There are three methods provided for extracting indented text blocks:
-    
+
     - `get_indented()`: use when the indent is unknown.
     - `get_known_indented()`: use when the indent is known for all lines.
     - `get_first_known_indented()`: use when only the first line's indent is
@@ -1054,7 +1079,7 @@ class ViewList:
     child and parent lists can be broken by calling `disconnect()` on the
     child list.
 
-    Also, ViewList objects keep track of the source & offset of each item. 
+    Also, ViewList objects keep track of the source & offset of each item.
     This information is accessible via the `source()`, `offset()`, and
     `info()` methods.
     """
@@ -1111,21 +1136,20 @@ class ViewList:
     def __len__(self): return len(self.data)
 
     # The __getitem__()/__setitem__() methods check whether the index
-    # is a slice first, since native list objects start supporting
-    # them directly in Python 2.3 (no exception is raised when
-    # indexing a list with a slice object; they just work).
+    # is a slice first, since indexing a native list with a slice object
+    # just works.
 
     def __getitem__(self, i):
-        if isinstance(i, _SliceType):
+        if isinstance(i, types.SliceType):
             assert i.step in (None, 1),  'cannot handle slice with stride'
             return self.__class__(self.data[i.start:i.stop],
                                   items=self.items[i.start:i.stop],
-                                  parent=self, parent_offset=i.start)
+                                  parent=self, parent_offset=i.start or 0)
         else:
             return self.data[i]
 
     def __setitem__(self, i, item):
-        if isinstance(i, _SliceType):
+        if isinstance(i, types.SliceType):
             assert i.step in (None, 1), 'cannot handle slice with stride'
             if not isinstance(item, ViewList):
                 raise TypeError('assigning non-ViewList to ViewList slice')
@@ -1133,8 +1157,8 @@ class ViewList:
             self.items[i.start:i.stop] = item.items
             assert len(self.data) == len(self.items), 'data mismatch'
             if self.parent:
-                self.parent[i.start + self.parent_offset
-                            : i.stop + self.parent_offset] = item
+                self.parent[(i.start or 0) + self.parent_offset
+                            : (i.stop or len(self)) + self.parent_offset] = item
         else:
             self.data[i] = item
             if self.parent:
@@ -1151,8 +1175,8 @@ class ViewList:
             del self.data[i.start:i.stop]
             del self.items[i.start:i.stop]
             if self.parent:
-                del self.parent[i.start + self.parent_offset
-                                : i.stop + self.parent_offset]
+                del self.parent[(i.start or 0) + self.parent_offset
+                                : (i.stop or len(self)) + self.parent_offset]
 
     def __add__(self, other):
         if isinstance(other, ViewList):
@@ -1294,6 +1318,16 @@ class ViewList:
         """Break link between this list and parent list."""
         self.parent = None
 
+    def xitems(self):
+        """Return iterator yielding (source, offset, value) tuples."""
+        for (value, (source, offset)) in zip(self.data, self.items):
+            yield (source, offset, value)
+
+    def pprint(self):
+        """Print the list in `grep` format (`source:offset:value` lines)"""
+        for line in self.xitems():
+            print "%s:%d:%s" % line
+
 
 class StringList(ViewList):
 
@@ -1400,6 +1434,30 @@ class StringList(ViewList):
             block.data = [line[indent:] for line in block.data]
         return block
 
+    def pad_double_width(self, pad_char):
+        """
+        Pad all double-width characters in self by appending `pad_char` to each.
+        For East Asian language support.
+        """
+        if hasattr(unicodedata, 'east_asian_width'):
+            east_asian_width = unicodedata.east_asian_width
+        else:
+            return                      # new in Python 2.4
+        for i in range(len(self.data)):
+            line = self.data[i]
+            if isinstance(line, unicode):
+                new = []
+                for char in line:
+                    new.append(char)
+                    if east_asian_width(char) in 'WF': # 'W'ide & 'F'ull-width
+                        new.append(pad_char)
+                self.data[i] = ''.join(new)
+
+    def replace(self, old, new):
+        """Replace all occurrences of substring `old` with `new`."""
+        for i in range(len(self.data)):
+            self.data[i] = self.data[i].replace(old, new)
+
 
 class StateMachineError(Exception): pass
 class UnknownStateError(StateMachineError): pass
@@ -1433,7 +1491,8 @@ class StateCorrection(Exception):
 def string2lines(astring, tab_width=8, convert_whitespace=0,
                  whitespace=re.compile('[\v\f]')):
     """
-    Return a list of one-line strings with tabs expanded and no newlines.
+    Return a list of one-line strings with tabs expanded, no newlines, and
+    trailing whitespace stripped.
 
     Each tab is expanded with between 1 and `tab_width` spaces, so that the
     next character's index becomes a multiple of `tab_width` (8 by default).
@@ -1446,7 +1505,7 @@ def string2lines(astring, tab_width=8, convert_whitespace=0,
     """
     if convert_whitespace:
         astring = whitespace.sub(' ', astring)
-    return [s.expandtabs(tab_width) for s in astring.splitlines()]
+    return [s.expandtabs(tab_width).rstrip() for s in astring.splitlines()]
 
 def _exception_data():
     """
