@@ -1,4 +1,4 @@
-# $Id: __init__.py 5504 2008-01-24 13:53:14Z goodger $
+# $Id: __init__.py 6315 2010-04-28 12:28:33Z milde $
 # Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
@@ -19,13 +19,12 @@ import os
 import os.path
 import time
 import re
-from types import ListType
 try:
     import Image                        # check for the Python Imaging Library
 except ImportError:
     Image = None
 import docutils
-from docutils import frontend, nodes, utils, writers, languages
+from docutils import frontend, nodes, utils, writers, languages, io
 from docutils.transforms import writer_aux
 
 
@@ -53,25 +52,24 @@ class Writer(writers.Writer):
           % default_template_path,
           ['--template'],
           {'default': default_template_path, 'metavar': '<file>'}),
-        ('Specify a stylesheet URL, used verbatim.  Overrides '
-          '--stylesheet-path.',
+        ('Specify comma separated list of stylesheet URLs. '
+          'Overrides previous --stylesheet and --stylesheet-path settings.',
           ['--stylesheet'],
           {'metavar': '<URL>', 'overrides': 'stylesheet_path'}),
-         ('Specify a stylesheet file, relative to the current working '
-          'directory.  The path is adjusted relative to the output HTML '
-          'file.  Overrides --stylesheet.  Default: "%s"'
-          % default_stylesheet_path,
+         ('Specify comma separated list of stylesheet paths. '
+          'With --link-stylesheet, '
+          'the path is rewritten relative to the output HTML file. '
+          'Default: "%s"' % default_stylesheet_path,
           ['--stylesheet-path'],
           {'metavar': '<file>', 'overrides': 'stylesheet',
            'default': default_stylesheet_path}),
-         ('Embed the stylesheet in the output HTML file.  The stylesheet '
-          'file must be accessible during processing (--stylesheet-path is '
-          'recommended).  This is the default.',
+         ('Embed the stylesheet(s) in the output HTML file.  The stylesheet '
+          'files must be accessible during processing. This is the default.',
           ['--embed-stylesheet'],
           {'default': 1, 'action': 'store_true',
            'validator': frontend.validate_boolean}),
-         ('Link to the stylesheet in the output HTML file.  Default: '
-          'embed the stylesheet, do not link to it.',
+         ('Link to the stylesheet(s) in the output HTML file. '
+          'Default: embed stylesheets.',
           ['--link-stylesheet'],
           {'dest': 'embed_stylesheet', 'action': 'store_false'}),
          ('Specify the initial header level.  Default is 1 for "<h1>".  '
@@ -120,6 +118,10 @@ class Writer(writers.Writer):
          ('Disable compact simple field lists.',
           ['--no-compact-field-lists'],
           {'dest': 'compact_field_lists', 'action': 'store_false'}),
+         ('Added to standard table classes. '
+          'Defined styles: "borderless". Default: ""',
+          ['--table-style'],
+          {'default': ''}),
          ('Omit the XML declaration.  Use with caution.',
           ['--no-xml-declaration'],
           {'dest': 'xml_declaration', 'default': 1, 'action': 'store_false',
@@ -158,7 +160,7 @@ class Writer(writers.Writer):
         self.output = self.apply_template()
 
     def apply_template(self):
-        template_file = open(self.document.settings.template)
+        template_file = open(self.document.settings.template, 'rb')
         template = unicode(template_file.read(), 'utf-8')
         template_file.close()
         subs = self.interpolation_dict()
@@ -235,6 +237,7 @@ class HTMLTranslator(nodes.NodeVisitor):
     stylesheet_link = '<link rel="stylesheet" href="%s" type="text/css" />\n'
     embedded_stylesheet = '<style type="text/css">\n\n%s\n</style>\n'
     words_and_spaces = re.compile(r'\S+| +|\n')
+    sollbruchstelle = re.compile(r'.+\W\W.+|[-?].+', re.U) # wrap point inside word
 
     def __init__(self, document):
         nodes.NodeVisitor.__init__(self, document)
@@ -254,18 +257,19 @@ class HTMLTranslator(nodes.NodeVisitor):
                                  self.head_prefix_template % (lcode, lcode)])
         self.html_prolog.append(self.doctype)
         self.head = self.meta[:]
-        stylesheet = utils.get_stylesheet_reference(settings)
-        self.stylesheet = []
-        if stylesheet:
-            if settings.embed_stylesheet:
-                stylesheet = utils.get_stylesheet_reference(
-                    settings, os.path.join(os.getcwd(), 'dummy'))
-                settings.record_dependencies.add(stylesheet)
-                stylesheet_text = open(stylesheet).read()
-                self.stylesheet = [self.embedded_stylesheet % stylesheet_text]
-            else:
-                self.stylesheet = [self.stylesheet_link
-                                   % self.encode(stylesheet)]
+        # stylesheets
+        styles = utils.get_stylesheet_list(settings)
+        if settings.stylesheet_path and not(settings.embed_stylesheet):
+            styles = [utils.relative_path(settings._destination, sheet)
+                      for sheet in styles]
+        if settings.embed_stylesheet:
+            settings.record_dependencies.add(*styles)
+            self.stylesheet = [self.embedded_stylesheet %
+                io.FileInput(source_path=sheet, encoding='utf-8').read()
+                for sheet in styles]
+        else: # link to stylesheets
+            self.stylesheet = [self.stylesheet_link % self.encode(stylesheet)
+                               for stylesheet in styles]
         self.body_prefix = ['</head>\n<body>\n']
         # document title, subtitle display
         self.body_pre_docinfo = []
@@ -307,14 +311,15 @@ class HTMLTranslator(nodes.NodeVisitor):
     def encode(self, text):
         """Encode special characters in `text` & return."""
         # @@@ A codec to do these and all other HTML entities would be nice.
-        text = text.replace("&", "&amp;")
-        text = text.replace("<", "&lt;")
-        text = text.replace('"', "&quot;")
-        text = text.replace(">", "&gt;")
-        text = text.replace("@", "&#64;") # may thwart some address harvesters
-        # Replace the non-breaking space character with the HTML entity:
-        text = text.replace(u'\u00a0', "&nbsp;")
-        return text
+        text = unicode(text)
+        return text.translate({
+            ord('&'): u'&amp;',
+            ord('<'): u'&lt;',
+            ord('"'): u'&quot;',
+            ord('>'): u'&gt;',
+            ord('@'): u'&#64;', # may thwart some address harvesters
+            # TODO: convert non-breaking space only if needed?
+            0xa0: u'&nbsp;'}) # non-breaking space
 
     def cloak_mailto(self, uri):
         """Try to hide a mailto: URL from harvesters."""
@@ -353,13 +358,13 @@ class HTMLTranslator(nodes.NodeVisitor):
         for (name, value) in attributes.items():
             atts[name.lower()] = value
         classes = node.get('classes', [])
-        if atts.has_key('class'):
+        if 'class' in atts:
             classes.append(atts['class'])
         if classes:
             atts['class'] = ' '.join(classes)
-        assert not atts.has_key('id')
+        assert 'id' not in atts
         ids.extend(node.get('ids', []))
-        if atts.has_key('ids'):
+        if 'ids' in atts:
             ids.extend(atts['ids'])
             del atts['ids']
         if ids:
@@ -384,7 +389,7 @@ class HTMLTranslator(nodes.NodeVisitor):
             # value=None was used for boolean attributes without
             # value, but this isn't supported by XHTML.
             assert value is not None
-            if isinstance(value, ListType):
+            if isinstance(value, list):
                 values = [unicode(v) for v in value]
                 parts.append('%s="%s"' % (name.lower(),
                                           self.attval(' '.join(values))))
@@ -731,9 +736,9 @@ class HTMLTranslator(nodes.NodeVisitor):
             tagname = 'td'
             del atts['class']
         node.parent.column += 1
-        if node.has_key('morerows'):
+        if 'morerows' in node:
             atts['rowspan'] = node['morerows'] + 1
-        if node.has_key('morecols'):
+        if 'morecols' in node:
             atts['colspan'] = node['morecols'] + 1
             node.parent.column += node['morecols']
         self.body.append(self.starttag(node, tagname, '', **atts))
@@ -752,9 +757,9 @@ class HTMLTranslator(nodes.NodeVisitor):
         usable.
         """
         atts = {}
-        if node.has_key('start'):
+        if 'start' in node:
             atts['start'] = node['start']
-        if node.has_key('enumtype'):
+        if 'enumtype' in node:
             atts['class'] = node['enumtype']
         # @@@ To do: prefix, suffix. How? Change prefix/suffix to a
         # single "format" attribute? Use CSS2?
@@ -807,7 +812,8 @@ class HTMLTranslator(nodes.NodeVisitor):
                             if not isinstance(n, nodes.Invisible)]
                 if not (len(children) == 0 or
                         len(children) == 1 and
-                        isinstance(children[0], nodes.paragraph)):
+                        isinstance(children[0],
+                                   (nodes.paragraph, nodes.line_block))):
                     self.compact_field_list = 0
                     break
         self.body.append(self.starttag(node, 'table', frame='void',
@@ -842,9 +848,9 @@ class HTMLTranslator(nodes.NodeVisitor):
     def visit_figure(self, node):
         atts = {'class': 'figure'}
         if node.get('width'):
-            atts['style'] = 'width: %spx' % node['width']
+            atts['style'] = 'width: %s' % node['width']
         if node.get('align'):
-            atts['align'] = node['align']
+            atts['class'] += " align-" + node['align']
         self.body.append(self.starttag(node, 'div', **atts))
 
     def depart_figure(self, node):
@@ -940,27 +946,37 @@ class HTMLTranslator(nodes.NodeVisitor):
 
     def visit_image(self, node):
         atts = {}
-        atts['src'] = node['uri']
-        if node.has_key('width'):
+        uri = node['uri']
+        # place SVG and SWF images in an <object> element
+        types = {'.svg': 'image/svg+xml',
+                 '.swf': 'application/x-shockwave-flash'}
+        ext = os.path.splitext(uri)[1].lower()
+        if ext in ('.svg', '.swf'):
+            atts['data'] = uri
+            atts['type'] = types[ext]
+        else:
+            atts['src'] = uri
+            atts['alt'] = node.get('alt', uri)
+        # image size
+        if 'width' in node:
             atts['width'] = node['width']
-        if node.has_key('height'):
+        if 'height' in node:
             atts['height'] = node['height']
-        if node.has_key('scale'):
-            if Image and not (node.has_key('width')
-                              and node.has_key('height')):
+        if 'scale' in node:
+            if Image and not ('width' in node and 'height' in node):
                 try:
-                    im = Image.open(str(atts['src']))
+                    im = Image.open(str(uri))
                 except (IOError, # Source image can't be found or opened
                         UnicodeError):  # PIL doesn't like Unicode paths.
                     pass
                 else:
-                    if not atts.has_key('width'):
+                    if 'width' not in atts:
                         atts['width'] = str(im.size[0])
-                    if not atts.has_key('height'):
+                    if 'height' not in atts:
                         atts['height'] = str(im.size[1])
                     del im
             for att_name in 'width', 'height':
-                if atts.has_key(att_name):
+                if att_name in atts:
                     match = re.match(r'([0-9.]+)(\S*)$', atts[att_name])
                     assert match
                     atts[att_name] = '%s%s' % (
@@ -968,7 +984,7 @@ class HTMLTranslator(nodes.NodeVisitor):
                         match.group(2))
         style = []
         for att_name in 'width', 'height':
-            if atts.has_key(att_name):
+            if att_name in atts:
                 if re.match(r'^[0-9.]+$', atts[att_name]):
                     # Interpret unitless values as pixels.
                     atts[att_name] += 'px'
@@ -976,7 +992,6 @@ class HTMLTranslator(nodes.NodeVisitor):
                 del atts[att_name]
         if style:
             atts['style'] = ' '.join(style)
-        atts['alt'] = node.get('alt', atts['src'])
         if (isinstance(node.parent, nodes.TextElement) or
             (isinstance(node.parent, nodes.reference) and
              not isinstance(node.parent.parent, nodes.TextElement))):
@@ -984,20 +999,15 @@ class HTMLTranslator(nodes.NodeVisitor):
             suffix = ''
         else:
             suffix = '\n'
-        if node.has_key('align'):
-            if node['align'] == 'center':
-                # "align" attribute is set in surrounding "div" element.
-                self.body.append('<div align="center" class="align-center">')
-                self.context.append('</div>\n')
-                suffix = ''
-            else:
-                # "align" attribute is set in "img" element.
-                atts['align'] = node['align']
-                self.context.append('')
+        if 'align' in node:
             atts['class'] = 'align-%s' % node['align']
+        self.context.append('')
+        if ext in ('.svg', '.swf'): # place in an object element,
+            # do NOT use an empty tag: incorrect rendering in browsers
+            self.body.append(self.starttag(node, 'object', suffix, **atts) +
+                             node.get('alt', uri) + '</object>' + suffix)
         else:
-            self.context.append('')
-        self.body.append(self.emptytag(node, 'img', suffix, **atts))
+            self.body.append(self.emptytag(node, 'img', suffix, **atts))
 
     def depart_image(self, node):
         self.body.append(self.context.pop())
@@ -1052,9 +1062,13 @@ class HTMLTranslator(nodes.NodeVisitor):
         text = node.astext()
         for token in self.words_and_spaces.findall(text):
             if token.strip():
-                # Protect text like "--an-option" from bad line wrapping:
-                self.body.append('<span class="pre">%s</span>'
-                                 % self.encode(token))
+                # Protect text like "--an-option" and the regular expression
+                # ``[+]?(\d+(\.\d*)?|\.\d+)`` from bad line wrapping
+                if self.sollbruchstelle.search(token):
+                    self.body.append('<span class="pre">%s</span>'
+                                     % self.encode(token))
+                else:
+                    self.body.append(self.encode(token))
             elif token in ('\n', ' '):
                 # Allow breaks at whitespace:
                 self.body.append(token)
@@ -1210,7 +1224,7 @@ class HTMLTranslator(nodes.NodeVisitor):
 
     def visit_reference(self, node):
         atts = {'class': 'reference'}
-        if node.has_key('refuri'):
+        if 'refuri' in node:
             atts['href'] = node['refuri']
             if ( self.settings.cloak_email_addresses
                  and atts['href'].startswith('mailto:')):
@@ -1218,7 +1232,7 @@ class HTMLTranslator(nodes.NodeVisitor):
                 self.in_mailto = 1
             atts['class'] += ' external'
         else:
-            assert node.has_key('refid'), \
+            assert 'refid' in node, \
                    'References must have "refuri" or "refid" attribute.'
             atts['href'] = '#' + node['refid']
             atts['class'] += ' internal'
@@ -1357,15 +1371,16 @@ class HTMLTranslator(nodes.NodeVisitor):
         self.body.append('</div>\n')
 
     def visit_table(self, node):
+        classes = ' '.join(['docutils', self.settings.table_style]).strip()
         self.body.append(
-            self.starttag(node, 'table', CLASS='docutils', border="1"))
+            self.starttag(node, 'table', CLASS=classes, border="1"))
 
     def depart_table(self, node):
         self.body.append('</table>\n')
 
     def visit_target(self, node):
-        if not (node.has_key('refuri') or node.has_key('refid')
-                or node.has_key('refname')):
+        if not ('refuri' in node or 'refid' in node
+                or 'refname' in node):
             self.body.append(self.starttag(node, 'span', '', CLASS='target'))
             self.context.append('</span>')
         else:
