@@ -1,4 +1,4 @@
-# $Id: frontmatter.py 4564 2006-05-21 20:44:42Z wiemann $
+# $Id: frontmatter.py 5618 2008-07-28 08:37:32Z strank $
 # Author: David Goodger, Ueli Schlaepfer <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
@@ -388,7 +388,7 @@ class DocInfo(Transform):
             try:
                 name = field[0][0].astext()
                 normedname = nodes.fully_normalize_name(name)
-                if not (len(field) == 2 and bibliofields.has_key(normedname)
+                if not (len(field) == 2 and normedname in bibliofields
                         and self.check_empty_biblio_field(field, name)):
                     raise TransformError
                 canonical = bibliofields[normedname]
